﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Reflection;

namespace HelloWorld
{

    class Program
    {
        [Import("App")]
        public MyApp MainController { get; set; }

        [Import("type")]
        public TypeManager Type { get; set; }

        static void Main(string[] args)
        {
            Program app = new Program();
            app.Compose();

            app.Run();

            Console.ReadLine();
        }

        private void Run()
        {
            MainController.ListLocations();
            Type.PrintTypes();
        }

        public void Compose()
        {
            AssemblyCatalog assemblyCatalog = new AssemblyCatalog(Assembly.GetExecutingAssembly());
            CompositionContainer container = new CompositionContainer(assemblyCatalog);
            var batch = new CompositionBatch();
            batch.AddPart(this);
            container.Compose(batch);
        }
    }

    [Export("type")]
    internal class TypeManager
    {
        [Import]
        public IEnumerable<IShipper> Shippers { get; set; }

        public void PrintTypes()
        {
            foreach (var calc in Shippers)
            {
                Console.WriteLine(calc.GetType().Name);
            }
        }
    }

    [Export("App")]
    internal class MyApp
    {
        [Import]
        public IEnumerable<IShipper> Shippers { get; set; }

        public void ListLocations()
        {
            foreach (var shipper in Shippers)
            {
                Console.WriteLine(shipper.Location());
            }
        }
    }

    public interface IShipper
    {
        string Location();
    }

    [Export(typeof(IShipper))]
    class CambridgeShipper : IShipper
    {
        public string Location()
        {
            return "Cambridge";
        }
    }

    [Export(typeof(IShipper))]
    class BelfastShipper : IShipper
    {
        public string Location()
        {
            return "Belfast";
        }
    }
}
