using System.ComponentModel.Composition;
using System.ComponentModel.Composition.Hosting;
using System.Reflection;
using System.IO;

namespace MefEnabled
{
    public class Extender
    {
        [Import]
        public ILogger Logger { get; set; }

        public void Compose()
        {
            AggregateCatalog catalog = new AggregateCatalog();
            AssemblyCatalog assemblyCatalog = new AssemblyCatalog(Assembly.GetExecutingAssembly());
            DirectoryCatalog directoryCatalog = new DirectoryCatalog(Path.Combine(Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location),
                                                                          "Generators"), true);
            catalog.Catalogs.Add(directoryCatalog);

            catalog.Catalogs.Add(assemblyCatalog);

            //Set the defaults....
            CatalogExportProvider mainProvider = new CatalogExportProvider(assemblyCatalog);
            CompositionContainer container = new CompositionContainer(directoryCatalog, mainProvider);
            mainProvider.SourceProvider = container;

            var batch = new CompositionBatch();
            batch.AddPart(this);
            container.Compose(batch);

            Logger.Write("Compose complete");
        }
    }
}