using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using MefEnabled.Interfaces;

namespace MefEnabled
{
    [Export]
    public class GeneratorManager : INotifyImportSatisfaction
    {
        public string GetName(Type instance)
        {
            object[] generatorAttributes = instance.GetCustomAttributes(typeof(GeneratorAttribute), false);

            return ((GeneratorAttribute)generatorAttributes[0]).DisplayName;
        }

        [Import(AllowRecomposition = true)] 
        public List<IGenerator> Generators {get;set;}

        [Import]
        public ILogger Logger { get; set; }


           [Import]
           public ExportCollection<ILogger, ILoggerCapabilities> ExtraLoggers { get; set; }

        public delegate void GeneratorLoadedEventHandler(object source, EventArgs args);
        public event GeneratorLoadedEventHandler GeneratorLoaded; 
        public void ImportCompleted()
        {
            foreach (var export in ExtraLoggers)
            {
                if (export.MetadataView.Store == Store.File)
                    export.GetExportedObject().Write("Log to file");   
            }

            Logger.Write("Import has completed");
            if (GeneratorLoaded != null)
                GeneratorLoaded(this, EventArgs.Empty);
        }
    }
}