﻿using System;
using System.ComponentModel.Composition;
using System.IO;

namespace MefEnabled
{
    public interface ILogger
    {
        void Write(string message);
    }

    [Export(typeof(ILogger))]
    public class Logger : ILogger
    {
        public void Write(string message)
        {
            Console.WriteLine(message);
        }
    }

   public interface ILoggerCapabilities
   {
      Store Store { get; set; }
   }

   [MetadataAttribute]
    [AttributeUsage(AttributeTargets.Class)]
    public class LoggerCapabilities : Attribute, ILoggerCapabilities
   {
        public Store Store { get; set; }
    }
    public enum Store
    {
        Memory,
        Console,
        File
    }


    [Export(typeof(ILogger))]
    [LoggerCapabilities(Store = Store.File)]
    public class PrivateLogger : ILogger
    {
        StreamWriter writer;

        public PrivateLogger()
        {
            writer = new StreamWriter(Path.GetTempFileName());
        }

        public void Write(string message)
        {
            writer.Write(message);
            writer.Flush();
        }
    }
}
