using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;

namespace MEF_Old20Approach
{
    public class GeneratorManager
    {
        public string GetName(Type instance)
        {
            object[] generatorAttributes = instance.GetCustomAttributes(typeof(GeneratorAttribute), false);

            return ((GeneratorAttribute)generatorAttributes[0]).DisplayName;
        }

        private List<Type> generators = new List<Type>();

        public void LoadExternalGeneratorAssemblies()
        {
            // Get a list of all the dlls in the generator directory
            List<string> dlls = new List<string>();

            string pathToAssemblies = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
            pathToAssemblies = Path.Combine(pathToAssemblies, "Generators");
            if (Directory.Exists(pathToAssemblies))
            {
                foreach (string file in Directory.GetFiles(pathToAssemblies, "*.dll"))
                {
                    dlls.Add(file);
                }
            }

            Type generatorType = typeof(IGenerator);
            Type generatorAttributeType = typeof(GeneratorAttribute);
            foreach (string dll in dlls)
            {
                Assembly assembly = Assembly.LoadFile(dll);
                foreach (Type type in assembly.GetTypes())
                {
                    if (!type.IsPublic)
                        continue;

                    if (generatorType.IsAssignableFrom(type))
                    {
                        object[] generatorAttributes = type.GetCustomAttributes(generatorAttributeType, false);

                        if (generatorAttributes != null &&
                            generatorAttributes.Length == 1)
                        {
                            generators.Add(type);
                        }
                    }
                }
            }
        }

        public List<Type> GetGenerators()
        {
            return generators;
        }
    }
}