﻿using System;
using System.Collections.Generic;
using System.ComponentModel.Composition;
using System.Windows.Forms;
using MefEnabled.Interfaces;

namespace MefEnabled
{
    [Export]
    public partial class GeneratorSelector : UserControl
    {
        public GeneratorManager Manager { get; set; }

        public delegate void SelectedGeneratorEventHandler(object source, IGenerator generator);
        public event SelectedGeneratorEventHandler GeneratorChanged;

        [Import]
        public ILogger Logger { get; set; }

        [ImportingConstructor]
        public GeneratorSelector(GeneratorManager manager)
        {
            Manager = manager;
            InitializeComponent();

            manager.GeneratorLoaded += manager_GeneratorLoaded;
            PopulateCombo();
        }

        delegate void SetTextCallback();
        void manager_GeneratorLoaded(object source, EventArgs args)
        {
            if (generators.InvokeRequired)
            {
                Logger.Write("Invoke required");
                SetTextCallback d = ClearCombo;
                this.Invoke(d, new object[] { });
            }
            else
            {
                ClearCombo();
            }
        }

        private void ClearCombo()
        {
            generators.Items.Clear();
            PopulateCombo();
        }

        private void PopulateCombo()
        {
            foreach (var type in Manager.Generators)
            {
                GeneratorItem item = new GeneratorItem() {Generator = type, Name = Manager.GetName(type.GetType())};
                generators.Items.Add(item);
            }

            Logger.Write(string.Format("combo populated with {0}", generators.Items.Count));
        }

        private void generators_SelectedIndexChanged(object sender, EventArgs e)
        {
            GeneratorItem generatorItem = generators.SelectedItem as GeneratorItem;
            IGenerator instance = generatorItem.Generator;
            if (instance == null)
                return;

            if(GeneratorChanged != null)
                GeneratorChanged(this, instance);
        }

        class GeneratorItem
        {
            public string Name { get; set; }

            public IGenerator Generator { get; set; }

            public override string ToString()
            {
                return Name;
            }
        }
    }
}